// Copyright (C) 2008 David Bremner
//
// Author: David Bremner (bremner@unb.ca)
// Version: $Id$
//
// This file is part of lpmargin.
//
// lpmargin is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
//
// lpmargin is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with lpmargin.  If not, see <http://www.gnu.org/licenses/>.
#ifndef _DB_LP_H
#define _DB_LP_H
// Linear programs 
// min c 
//  s.t. 
// Ax=0, Bx>=0

#include <iostream>
#include <fstream>
#include <gmpxx.h>
extern "C"{
#ifndef GMP
#define GMP
#endif
#include "lrslib.h"
#undef copy
}
#include "QVector.h"
#include "QMatrix.h"

typedef enum { LRSLIB, CPLEX, GLPK } lp_lib_enum;

class LP{
 private:

  QVector c;
  QMatrix A;
  QMatrix B;

  QVector solution;
  Rational obj_val;

  bool solver_verbose;
  bool solver_debug;

  bool lplib_initialized;
  lp_lib_enum lp_lib;

  lrs_dat* lrs_data;
  lrs_dic* lrs_dict;

  void init_solver_lrs(void);
  void set_lrs_row(int row_index,const QVector &row,bool is_ineq);
  void solve_with_lrs(void);

  void fatal(const char *);

  
 public:

  LP(){ 
    lrs_data=0; 
    lrs_dict=0; 
    obj_val=0;
    solver_verbose=false;
    solver_debug=false;
    solution.clear();
    lp_lib=LRSLIB; 
    lplib_initialized=0; 
  }

  ~LP(){
    if (lrs_data || lrs_dict){
      fatal("Unfreed lrs data");
    }
  }

  void debug(){
    std::cerr << "DEBUG" << std::endl;
    solver_debug=true;
  }

  void set_eq(QMatrix eq){
    A=eq;
  }

  void set_ieq(QMatrix ieq){
    B=ieq;
  }

  void set_obj(QVector obj){
    c=obj;
  }
  
  void solve();
   
  void verbose(){ solver_verbose=1; }
  Rational obj_value() { return obj_val; };
  const QVector & opt_vector() { return solution; };
};
#endif
