// Copyright (C) 2008 David Bremner
//
// Author: David Bremner (bremner@unb.ca)
// Version: $Id$
//
// This file is part of lpmargin.
//
// lpmargin is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
//
// lpmargin is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with lpmargin.  If not, see <http://www.gnu.org/licenses/>.
#ifndef _DB_RANDOM_H
#define _DB_RANDOM_H
#include <iostream>
#include <string>
#include <sstream>
#include <limits>
#include <inttypes.h>
// Random number generator by Luc Devroye, translitered to C++ by db;

class Random {
  private:
  int32_t seed1,seed2;

  static const double factor = 1.0/2147483399.0;  
  
 public: 
  Random(int32_t seed1val=55555,int32_t seed2val=99999) { 
    set_seeds(seed1val,seed2val);
  }

  int32_t get_seed1() const { return seed1;};
  int32_t get_seed2() const { return seed2;};
  double  next();
  int32_t lnext () { 
    return static_cast<int32_t>(next()*std::numeric_limits<int32_t>::max());
  };
  void set_seeds(int32_t seed1val,int32_t seed2val){
    seed1=seed1val;
    seed2=seed2val;
  }

  void set_seeds(const char* seed_string) {
      
    std::istringstream seeds(seed_string);
    int32_t s1,s2;
    
    seeds >> s1;
    seeds.ignore(); // skip delimiter
    seeds >> s2;
    
    set_seeds(s1,s2);
      
  }


};

inline std::ostream & operator<< (std::ostream & os , const Random & rg){
 os << rg.get_seed1() << "," << rg.get_seed2() ;
 return os;
}

    
#endif
