// Copyright (C) 2008 David Bremner
//
// Author: David Bremner (bremner@unb.ca)
// Version: $Id$
//
// This file is part of lpmargin.
//
// lpmargin is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
//
// lpmargin is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with lpmargin.  If not, see <http://www.gnu.org/licenses/>.
#ifndef _DB_BASEVEC_H
#define _DB_BASEVEC_H

#include <vector>

// define a base clase with operations common to Vectors and Matrices
 
template <class T>
class vecbase : public std::vector<T> {
  typedef  T val;
  typedef  const val& crval;

  typedef  vecbase<T>  vec;
  typedef  const vec& crvec;

  typedef  std::vector<T>  svec;
  public:
  
  
  vecbase<T>(): svec() { };
  vecbase<T>(int num) : svec(num) {  };
  vecbase<T>(crvec v) : svec(v) { };
  vecbase<T>(int num, val x) : svec(num,x) {  };

  void append(crval x) { push_back(x);};
  vec join(crval x) { vec w(*this);  w.append(x); return (w);};
  virtual vec join(crvec v) { vec w(*this);  w.append(v); return (w);};
  
  vec& append(crvec v) { 
    typename vec::const_iterator j; 
    for (j=v.begin(); j!=v.end(); j++)
      append(*j);
    return *this;
  };

  vec& operator |= (crvec v) { return append(v); };
  vec& operator |= (crval x) { return append(x); };
  vec operator | (crvec v) { return join(v); };
  vec operator | (crval x) { return join(x); };

};




#endif
